package org.mirkorusso.K2KC45;

import java.io.File;
import java.io.IOException;

import org.apache.commons.io.FileDeleteStrategy;
import org.apache.commons.io.FileUtils;
import org.knime.core.data.DataColumnDomain;
import org.knime.core.data.DataColumnSpec;
import org.knime.core.data.DataTableSpec;
import org.knime.core.node.BufferedDataTable;
import org.knime.core.node.CanceledExecutionException;
import org.knime.core.node.defaultnodesettings.SettingsModelBoolean;
import org.knime.core.node.defaultnodesettings.SettingsModelDouble;
import org.knime.core.node.defaultnodesettings.SettingsModelInteger;
import org.knime.core.node.ExecutionContext;
import org.knime.core.node.ExecutionMonitor;
import org.knime.core.node.InvalidSettingsException;
import org.knime.core.node.NodeModel;
import org.knime.core.node.NodeSettingsRO;
import org.knime.core.node.NodeSettingsWO;
import keel.Algorithms.Decision_Trees.C45.C45;


/**
 * This is the model implementation of K2KNodeModel.
 * 
 *
 * @author Mirko Russo
 */
public class K2KC45NodeModel extends NodeModel {  
	
	static final String BOOL = "pruned";
	static final String DBL = "confidence";
	static final String INT = "instancesPerLeaf";
	static final String MAIN_DIRECTORY = "./K2KNodeTempFiles/";
	static final String CONFIG_FILE_NAME = "config.txt";
	private static final String ALGORITHM_NAME = "C4.5 Decision Tree";
	private String DatasetName = null;

    private final SettingsModelBoolean m_bool =
            new SettingsModelBoolean(BOOL, true);

    private final SettingsModelDouble m_dbl =
            new SettingsModelDouble(DBL, .25);

    private final SettingsModelInteger m_int =
            new SettingsModelInteger(INT, 2);
 

    protected K2KC45NodeModel() {
        super(2, 2);
    }
    

	protected BufferedDataTable[] execute(final BufferedDataTable[] inData, 
    		final ExecutionContext exec) throws Exception {
		try {
			super.finalize();
		} catch (Throwable e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    	DataTableSpec inputTableSpec = inData[0].getDataTableSpec();
    	int numColumns = inputTableSpec.getNumColumns();
    	DatasetName = inputTableSpec.getName().substring(0, inputTableSpec.getName().length() - 4);
    	
    	String DataFilesDirectory = "./Datasets/";
    	String TrainingFileName = DatasetName+"-tra.dat";
    	String TestFileName = DatasetName+"-tst.dat";
    	String ResultsFileDirectory = "./Results/";
    	
    	File FolderCreator = new File(MAIN_DIRECTORY);
    	FolderCreator.mkdir();
    	
    	FolderCreator = new File(MAIN_DIRECTORY + DataFilesDirectory);
    	FolderCreator.mkdir();
    	
    	FolderCreator = new File(MAIN_DIRECTORY + ResultsFileDirectory);
    	FolderCreator.mkdir();
    	
    	DataColumnSpec[] dcs = new DataColumnSpec[numColumns];
    	for (int i = 0; i < numColumns; i++)
    		dcs[i] = inputTableSpec.getColumnSpec(i);
    	
    	DataColumnDomain[] dcd =  new DataColumnDomain[numColumns];
    	for (int i = 0; i < numColumns; i++)
    		dcd[i] = dcs[i].getDomain();
    	
    	KNIME2KEEL KnimetoKeel = new KNIME2KEEL();
    	KnimetoKeel.createConfigFile(DatasetName, ALGORITHM_NAME, BOOL, DBL, INT, MAIN_DIRECTORY, DataFilesDirectory, ResultsFileDirectory, m_bool, m_dbl, m_int);
    	KnimetoKeel.createDataFiles(DatasetName, MAIN_DIRECTORY + DataFilesDirectory + TrainingFileName, numColumns, dcs, dcd, inData[0]);
    	KnimetoKeel.createDataFiles(DatasetName, MAIN_DIRECTORY + DataFilesDirectory + TestFileName, numColumns, dcs, dcd, inData[1]);

    	String[] args = {MAIN_DIRECTORY + CONFIG_FILE_NAME};
    	try
		{
			C45.main(args);
		} catch (Exception e)
		{
			e.printStackTrace();
			throw e;
		}
    	
    	KEEL2KNIME KeeltoKnime = new KEEL2KNIME();
    	BufferedDataTable[] outData = new BufferedDataTable[2];
    	outData[0] = KeeltoKnime.predictionsExtractor(MAIN_DIRECTORY + ResultsFileDirectory + "result.tra", inData[0], exec);
    	outData[1] = KeeltoKnime.predictionsExtractor(MAIN_DIRECTORY + ResultsFileDirectory + "result.tst", inData[1], exec);
    	
    	System.gc();
    	FileDeleteStrategy.FORCE.delete(new File(MAIN_DIRECTORY + "CONFIG_FILE_NAME"));
    	FileUtils.deleteDirectory(new File(MAIN_DIRECTORY));
   
    	return outData;
	}
    

    @Override
    protected void reset() {
    }


    @Override
    protected DataTableSpec[] configure(final DataTableSpec[] inSpecs)
            throws InvalidSettingsException {
        return new DataTableSpec[]{null, null};
    }


    @Override
    protected void saveSettingsTo(final NodeSettingsWO settings) {
        m_bool.saveSettingsTo(settings);
        m_dbl.saveSettingsTo(settings);
        m_int.saveSettingsTo(settings);

    }


    @Override
    protected void loadValidatedSettingsFrom(final NodeSettingsRO settings)
            throws InvalidSettingsException {
        m_bool.loadSettingsFrom(settings);
        m_dbl.loadSettingsFrom(settings);
        m_int.loadSettingsFrom(settings);
    }
    
    
    @Override
    protected void validateSettings(final NodeSettingsRO settings)
            throws InvalidSettingsException {
        m_bool.validateSettings(settings);
        m_dbl.validateSettings(settings);
        m_int.validateSettings(settings);
    }


    @Override
    protected void loadInternals(final File internDir,
            final ExecutionMonitor exec) throws IOException,
            CanceledExecutionException {
    }

    
    @Override
    protected void saveInternals(final File internDir,
            final ExecutionMonitor exec) throws IOException,
            CanceledExecutionException {
    }
}