package org.mirkorusso.K2KC45;


import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import org.knime.core.data.DataColumnDomain;
import org.knime.core.data.DataColumnSpec;
import org.knime.core.data.DataRow;
import org.knime.core.node.BufferedDataTable;
import org.knime.core.node.defaultnodesettings.SettingsModelBoolean;
import org.knime.core.node.defaultnodesettings.SettingsModelDouble;
import org.knime.core.node.defaultnodesettings.SettingsModelInteger;


class KNIME2KEEL {
	void createConfigFile(String DatasetName, String AlgorithmName, String BOOL, String DBL, String INT,
			String MainDirectory, String DataFileDirectory, String ResultsFileDirectory, SettingsModelBoolean m_bool, 
			SettingsModelDouble m_dbl, SettingsModelInteger m_int) throws IOException {
		FileWriter configFile = new FileWriter(MainDirectory + "config.txt");
		BufferedWriter writer = new BufferedWriter(configFile);
        writer.write("algorithm = " + AlgorithmName + "\n");
        writer.write("inputData = \"" + MainDirectory  + DataFileDirectory + DatasetName + "-tra.dat\" \"" + MainDirectory  + DataFileDirectory + DatasetName + "-tra.dat\" \"" + MainDirectory + DataFileDirectory + DatasetName + "-tst.dat\"\n");
        writer.write("outputData = \"" + MainDirectory + ResultsFileDirectory + "result.tra\" \"" + MainDirectory + ResultsFileDirectory + "result.tst\" \"" + MainDirectory + ResultsFileDirectory + "result.txt\"\n\n"); 
        writer.write(BOOL + " = " + toUpperString(m_bool.getBooleanValue()) + "\n");
        writer.write(DBL + " = " + m_dbl.getDoubleValue() + "\n");
        writer.write(INT + " = " + m_int.getIntValue() + "\n");
        writer.close();
        configFile.close();
	}
	
	void createDataFiles(String DatasetName, String FilePath, 
			int numColumns, DataColumnSpec[] dcs, DataColumnDomain[] dcd, 
			BufferedDataTable inData) throws IOException {
		
		FileWriter dataFile = new FileWriter(FilePath);
		BufferedWriter writer = new BufferedWriter(dataFile);
        writer.write("@relation " + DatasetName + "\n");
        
        for(int i = 0; i < numColumns; i++)
        	writer.write("@attribute " + dcs[i].getName().replace(" ", "") + 
        			" " + getDataType(dcs[i]) + getDomain(dcs[i]) + "\n");
        
        String tmp = "@inputs ";
        
        for(int i = 0; i < numColumns; i++)
        	if (dcd[i].hasBounds()) 
        		tmp += dcs[i].getName().replace(" ", "") + ", ";
        
        while(tmp.endsWith(", "))
        	tmp=tmp.substring(0, tmp.length() - 2);
        
        writer.write(tmp + "\n");
        
        tmp = "@outputs ";
        
        for(int i = 0; i < numColumns; i++)
        	if (!dcd[i].hasBounds()) 
        		tmp += dcs[i].getName().replace(" ", "") + ", ";
        
        while(tmp.endsWith(", "))
        	tmp=tmp.substring(0, tmp.length() - 2);	

        writer.write(tmp + "\n");
        writer.write("@data\n");
        
        for (DataRow r: inData) {
            tmp = "";
            
        	for(int i = 0; i < numColumns; i++)
        		tmp += r.getCell(i).toString() + ", ";
        	
        	while(tmp.endsWith(", "))
            	tmp=tmp.substring(0, tmp.length() - 2);	
        	
        	writer.write(tmp + "\n");
        }
        writer.close();
        dataFile.close();
	}

	private String getDomain(DataColumnSpec dcs) {
		if(getDataType(dcs).equals("nominal")) 
			return " " + dcs.getDomain().getValues().toString().replace('[', '{').replace(']', '}');
		else
			return " [" + dcs.getDomain().getLowerBound().toString() + ", " + dcs.getDomain().getUpperBound().toString() + "]";
	}

	private String getDataType(DataColumnSpec dataColumnSpec) {
		String s = dataColumnSpec.getType().toString();
		
		if(s.equals("Number (double precision)"))
			return "real";
		else if(s.equals("Number (integer)") || s.equals("Number (long)" ))
			return "integer";
		else if(s.equals("String"))
			return "nominal";
		else
			return "nominal";
	}

	private String toUpperString(Boolean b) {
    	if (b)
    		return "TRUE";
    	else
    		return "FALSE";
    }
}
